/* TokenLst.c -- Module containing Token list ADT and token scanner
 *
 * See include file for details
 */
#include "tokenlst.h"
#include "stddefs.h"

#include <string.h>
#include <malloc.h>

#define	CreateString(length) ((char *) malloc(length))

/* -- Non publicly declared function prototypes -- */

/* char	*CreateString(int length); */
tokenList *CreateToken(void);


/* -- Publicly Declared functions -- */

/* -- Function GetTokens
 */
tokenList *GetTokens(char *text)
{
	bool 	inToken;						/* Are we currently in a token? */
   char	read,							/* Last read character */
   		*tokenBuffer,				/* Buffer for reading a token */
			*index;					   /* index into tokenBuffer */

   tokenList	*list;				/* Token list to return */

   if ((tokenBuffer = CreateString(MAXSTRING)) == NULL)	/* Allocate token buffer */
   	return NULL;

   list = NULL;							/* Reset token list */
   index = tokenBuffer;					/* Reset index */

   inToken = FALSE;		/* Reset flags */

   while (TRUE) {						/* All returns are from within loop */
   	read = *text++;

      switch (read) {
      	case EOS:							/* End of string */
            if (inToken) {
	         	*index = EOS;						/* Nul-terminate */
	            inToken = FALSE;					/* No longer in a token */
   	         index = tokenBuffer;				/* Reset index */

					list = AddToken(tokenBuffer, list);	/* Add token to list */
            }

            free (tokenBuffer);				/* Deallocate token buffer */

            return list;						/* Exit, return token list */
            /* break; */

         case ' ':							/* Whitespace */
         case TAB:
         	if (inToken) {
   	      	inToken = FALSE;					/* No longer in a token */
      	      *index = EOS;						/* Nul-terminate string */
         	   index = tokenBuffer;				/* Reset index */

            	list = AddToken(tokenBuffer, list);	/* Add token to list */
            } else
            	;										/* not in a token, so ignore */
            break;

         default:
 	      	if (!inToken)					/* If we're not in a token, then... */
     	      	inToken = TRUE;				/* start of a new token */

         	*index++ = read;			/* In a token, so copy */

            break;
      }
   }
}


/* -- Function AddToken
 */
tokenList *AddToken(char tokenText[], tokenList *list)
{
   tokenList	*insert,							/* index to insert option at */
   				*index;

   if ((insert = CreateToken()) == NULL)	/* Can we create a token? */
     	return NULL;

   strncpy(insert -> token, tokenText, MAXSTRING - 1);	/* Copy token text */

   index = list;									/* Reset index into token list */

   if (index != NULL) {							/* Is it a null list */
   	while (index -> NEXT != NULL)				/* Loop until end of list */
      	index = index -> NEXT;						/* Step through list */

      index -> NEXT = insert;						/* Insert token at end of list */
      return list;                           /* return list */
   }

   return insert;									/* Otherwise just return token (only node in list) */
}


/* -- Function DestroyTokenList
 */
tokenList *DestroyTokenList(tokenList *list)
{
	if (list != NULL) {							/* Is it not a null list? */
   	DestroyTokenList(list -> NEXT);			/* Recursive call */
      free (list -> token);						/* Free token text */
		free (list);									/* Free token structure */
	}
   return NULL;									/* return an empty list */
}


tokenList *ConcatenateTokenList(tokenList *list1, tokenList *list2)
{
	tokenList	*index;

	if (list2 == NULL)			/* No list 2 */
		return list1;
	if (list1 == NULL)			/* No list 1 */
		return list2;

	index = list1;

	while (index -> NEXT != NULL)		/* Shuffle to end of list */
		index = index -> NEXT;

	index -> NEXT = list2;

	return list1;				/* Return the resulting list */
}


int TokenListLength(tokenList *list)
{
	int	length = 0;

	while (list != NULL) {
		length++;
		list = list -> NEXT;
	}
	return length;
}

/* -- End of publicly declared functions -- */

/* --- Function CreateString
 * Pre: TRUE
 * Post: (Memory for a string of length 'length' was allocate dand returned) ||
 *			(There was insufficient memory && NULL was returned)
 *
char	*CreateString(int length)
{
	return (char *) malloc(length);
}*/


/* -- Function CreateToken
 * Pre: TRUE
 * Post: (A token was allocated and returned) ||
 *			(The operation was unsuccessful && NULL was returned)
*/
tokenList *CreateToken(void)
{
	tokenList	*tempToken;		/* Temporary token for allocation */

   if ((tempToken = (tokenList *) malloc(sizeof(tokenList))) == NULL)
   	return NULL;

   if ((tempToken -> token = CreateString(MAXSTRING)) == NULL) {
		free (tempToken);
      return NULL;
   }

	tempToken -> NEXT = NULL;

   return tempToken;				/* Send back token */
}

/* -- END of TokenLst.C -- */
